@@ -24,10 +24,11 @@
 #include <QDomDocument>
 #include <QCoreApplication>
 #include <QSet>
+#include <QRecursiveMutex>
+
 #include <functional>
 
 #include "qgis.h"
-#include "qgsunittypes.h"
 #include "qgsinterval.h"
 #include "qgsexpressionnode.h"
 
@@ -44,6 +45,91 @@ class QgsExpressionContext;
 class QgsExpressionPrivate;
 class QgsExpressionFunction;
 
+#ifndef SIP_RUN
+///@cond PRIVATE
+struct HelpArg
+{
+  HelpArg( const QString &arg, const QString &desc, bool descOnly = false, bool syntaxOnly = false,
+           bool optional = false, const QString &defaultVal = QString() )
+    : mArg( arg )
+    , mDescription( desc )
+    , mDescOnly( descOnly )
+    , mSyntaxOnly( syntaxOnly )
+    , mOptional( optional )
+    , mDefaultVal( defaultVal )
+  {}
+
+  QString mArg;
+  QString mDescription;
+  bool mDescOnly;
+  bool mSyntaxOnly;
+  bool mOptional;
+  QString mDefaultVal;
+};
+
+struct HelpExample
+{
+  HelpExample( const QString &expression, const QString &returns, const QString &note = QString() )
+    : mExpression( expression )
+    , mReturns( returns )
+    , mNote( note )
+  {}
+
+  QString mExpression;
+  QString mReturns;
+  QString mNote;
+};
+
+
+struct HelpVariant
+{
+  HelpVariant( const QString &name, const QString &description,
+               const QList<HelpArg> &arguments = QList<HelpArg>(),
+               bool variableLenArguments = false,
+               const QList<HelpExample> &examples = QList<HelpExample>(),
+               const QString &notes = QString(),
+               const QStringList &tags = QStringList() )
+    : mName( name )
+    , mDescription( description )
+    , mArguments( arguments )
+    , mVariableLenArguments( variableLenArguments )
+    , mExamples( examples )
+    , mNotes( notes )
+    , mTags( tags )
+  {}
+
+  QString mName;
+  QString mDescription;
+  QList<HelpArg> mArguments;
+  bool mVariableLenArguments;
+  QList<HelpExample> mExamples;
+  QString mNotes;
+  QStringList mTags;
+};
+
+
+struct Help
+{
+  Help() = default;
+
+  Help( const QString &name, const QString &type, const QString &description, const QList<HelpVariant> &variants )
+    : mName( name )
+    , mType( type )
+    , mDescription( description )
+    , mVariants( variants )
+  {}
+
+  QString mName;
+  QString mType;
+  QString mDescription;
+  QList<HelpVariant> mVariants;
+};
+
+typedef QHash<QString, Help> HelpTextHash;
+
+///@endcond PRIVATE
+#endif
+
 /**
  * \ingroup core
  * \brief Class for parsing and evaluation of expressions (formerly called "search strings").
@@ -106,7 +192,6 @@ class CORE_EXPORT QgsExpression
 
     /**
      * Details about any parser errors that were found when parsing the expression.
-     * \since QGIS 3.0
      */
     struct CORE_EXPORT ParserError
     {
@@ -177,14 +262,12 @@ class CORE_EXPORT QgsExpression
     /**
      * Automatically convert this expression to a string where requested.
      *
-     * \since QGIS 3.0
      */
     operator QString() const SIP_SKIP;
 
     /**
      * Create an empty expression.
      *
-     * \since QGIS 3.0
      */
     QgsExpression();
 
@@ -194,7 +277,6 @@ class CORE_EXPORT QgsExpression
      * Compares two expressions. The operator returns TRUE
      * if the expression string is equal.
      *
-     * \since QGIS 3.0
      */
     bool operator==( const QgsExpression &other ) const;
 
@@ -202,7 +284,6 @@ class CORE_EXPORT QgsExpression
      * Checks if this expression is valid.
      * A valid expression could be parsed but does not necessarily evaluate properly.
      *
-     * \since QGIS 3.0
      */
     bool isValid() const;
 
@@ -213,7 +294,6 @@ class CORE_EXPORT QgsExpression
 
     /**
      * Returns parser error details including location of error.
-     * \since QGIS 3.0
      */
     QList<QgsExpression::ParserError> parserErrors() const;
 
@@ -227,7 +307,6 @@ class CORE_EXPORT QgsExpression
     /**
      * Gets the expression ready for evaluation - find out column indexes.
      * \param context context for preparing expression
-     * \since QGIS 2.12
      */
     bool prepare( const QgsExpressionContext *context );
 
@@ -259,7 +338,6 @@ class CORE_EXPORT QgsExpression
      * is not affected by any previous calls to QgsExpression::prepare(),
      * or QgsExpressionNode::prepare().
      *
-     * \since QGIS 3.0
      */
     QSet<QString> referencedVariables() const;
 
@@ -316,7 +394,6 @@ class CORE_EXPORT QgsExpression
      * If you are seeking to use these functions to introspect an expression you must
      * take care to do this with an unprepared expression.
      *
-     * \since QGIS 3.0
      */
     QSet<int> referencedAttributeIndexes( const QgsFields &fields ) const;
 
@@ -328,7 +405,6 @@ class CORE_EXPORT QgsExpression
     /**
      * Evaluate the feature and return the result.
      * \note this method does not expect that prepare() has been called on this instance
-     * \since QGIS 2.12
      */
     QVariant evaluate();
 
@@ -336,7 +412,6 @@ class CORE_EXPORT QgsExpression
      * Evaluate the expression against the specified context and return the result.
      * \param context context for evaluating expression
      * \note prepare() should be called before calling this method.
-     * \since QGIS 2.12
      */
     QVariant evaluate( const QgsExpressionContext *context );
 
@@ -351,7 +426,6 @@ class CORE_EXPORT QgsExpression
      * Checks whether an expression consists only of a single field reference
      *
      * \see expressionToLayerFieldIndex()
-     * \since QGIS 2.9
      */
     bool isField() const;
 
@@ -389,14 +463,12 @@ class CORE_EXPORT QgsExpression
      * \param context optional expression context
      * \param errorMessage will be filled with any error message from the validation
      * \returns TRUE if string is a valid expression
-     * \since QGIS 2.12
      */
     static bool checkExpression( const QString &text, const QgsExpressionContext *context, QString &errorMessage SIP_OUT );
 
     /**
      * Set the expression string, will reset the whole internal structure.
      *
-     * \since QGIS 3.0
      */
     void setExpression( const QString &expression );
 
@@ -444,9 +516,8 @@ class CORE_EXPORT QgsExpression
      * \note distances are only converted when a geomCalculator() has been set
      * \see setDistanceUnits()
      * \see areaUnits()
-     * \since QGIS 2.14
      */
-    QgsUnitTypes::DistanceUnit distanceUnits() const;
+    Qgis::DistanceUnit distanceUnits() const;
 
     /**
      * Sets the desired distance units for calculations involving geomCalculator(), e.g., "$length" and "$perimeter".
@@ -455,18 +526,16 @@ class CORE_EXPORT QgsExpression
      * \note distances are only converted when a geomCalculator() has been set
      * \see distanceUnits()
      * \see setAreaUnits()
-     * \since QGIS 2.14
      */
-    void setDistanceUnits( QgsUnitTypes::DistanceUnit unit );
+    void setDistanceUnits( Qgis::DistanceUnit unit );
 
     /**
      * Returns the desired areal units for calculations involving geomCalculator(), e.g., "$area".
      * \note areas are only converted when a geomCalculator() has been set
      * \see setAreaUnits()
      * \see distanceUnits()
-     * \since QGIS 2.14
      */
-    QgsUnitTypes::AreaUnit areaUnits() const;
+    Qgis::AreaUnit areaUnits() const;
 
     /**
      * Sets the desired areal units for calculations involving geomCalculator(), e.g., "$area".
@@ -475,9 +544,8 @@ class CORE_EXPORT QgsExpression
      * \note areas are only converted when a geomCalculator() has been set
      * \see areaUnits()
      * \see setDistanceUnits()
-     * \since QGIS 2.14
      */
-    void setAreaUnits( QgsUnitTypes::AreaUnit unit );
+    void setAreaUnits( Qgis::AreaUnit unit );
 
     /**
      * This function replaces each expression between [% and %]
@@ -488,7 +556,6 @@ class CORE_EXPORT QgsExpression
      * \param context Expression context
      * \param distanceArea Optional QgsDistanceArea. If specified, the QgsDistanceArea is used for distance
      * and area conversion
-     * \since QGIS 2.12
      */
     static QString replaceExpressionText( const QString &action, const QgsExpressionContext *context,
                                           const QgsDistanceArea *distanceArea = nullptr );
@@ -510,7 +577,6 @@ class CORE_EXPORT QgsExpression
      * \returns evaluated double value, or fallback value
      * \note this method is inefficient for bulk evaluation of expressions, it is intended
      * for one-off evaluations only.
-     * \since QGIS 2.7
      */
     static double evaluateToDouble( const QString &text, double fallbackValue );
 
@@ -549,7 +615,6 @@ class CORE_EXPORT QgsExpression
 
     /**
      * Deletes all registered functions whose ownership have been transferred to the expression engine.
-     * \since QGIS 2.12
      */
     static void cleanRegisteredFunctions();
 
@@ -585,7 +650,6 @@ class CORE_EXPORT QgsExpression
      * \param value value to convert to a string representation
      * \see quotedString()
      * \see quotedColumnRef()
-     * \since QGIS 2.14
      */
     static QString quotedValue( const QVariant &value );
 
@@ -596,12 +660,28 @@ class CORE_EXPORT QgsExpression
      * \param type value type
      * \see quotedString()
      * \see quotedColumnRef()
-     * \since QGIS 2.14
      */
-    static QString quotedValue( const QVariant &value, QVariant::Type type );
+    static QString quotedValue( const QVariant &value, QMetaType::Type type );
+
+    /**
+     * Returns a string representation of a literal value, including appropriate
+     * quotations where required.
+     * \param value value to convert to a string representation
+     * \param type value type
+     * \see quotedString()
+     * \see quotedColumnRef()
+     * \deprecated QGIS 3.38. Use the method with a QMetaType::Type argument instead.
+     */
+    Q_DECL_DEPRECATED static QString quotedValue( const QVariant &value, QVariant::Type type ) SIP_DEPRECATED;
 
     //////
 
+#ifndef SIP_RUN
+    ///@cond PRIVATE
+    static HelpTextHash &functionHelpTexts();
+    ///@endcond PRIVATE
+#endif
+
     /**
      * Returns the help text for a specified function.
      * \param name function name
@@ -636,7 +716,6 @@ class CORE_EXPORT QgsExpression
      * \param variableName name of variable
      * \see helpText()
      * \see addVariableHelpText()
-     * \since QGIS 2.12
      */
     static QString variableHelpText( const QString &variableName );
 
@@ -647,7 +726,6 @@ class CORE_EXPORT QgsExpression
      * \param value current value of variable to show in help text
      * \see helpText()
      * \see variableHelpText()
-     * \since QGIS 3.0
      */
     static QString formatVariableHelp( const QString &description, bool showValue = true, const QVariant &value = QVariant() );
 
@@ -664,7 +742,6 @@ class CORE_EXPORT QgsExpression
      * \param htmlOutput set to TRUE to allow HTML formatting, or FALSE for plain text output
      * \param maximumPreviewLength define the maximum character length of the preview
      * \returns formatted string, may contain HTML formatting characters if \a htmlOutput is TRUE
-     * \since QGIS 2.14
      */
     static QString formatPreviewString( const QVariant &value, bool htmlOutput = true, int maximumPreviewLength = 60 );
 
@@ -675,9 +752,19 @@ class CORE_EXPORT QgsExpression
      * \param value the value of the field
      * \param fieldType the type of the field on the left side used to quote the value. If not given, the value type is used instead
      * \returns the expression to evaluate field equality
-     * \since QGIS 3.0
      */
-    static QString createFieldEqualityExpression( const QString &fieldName, const QVariant &value, QVariant::Type fieldType = QVariant::Type::Invalid );
+    static QString createFieldEqualityExpression( const QString &fieldName, const QVariant &value, QMetaType::Type fieldType = QMetaType::Type::UnknownType );
+
+    /**
+     * Create an expression allowing to evaluate if a field is equal to a
+     *  value. The value may be null.
+     * \param fieldName the name of the field
+     * \param value the value of the field
+     * \param fieldType the type of the field on the left side used to quote the value. If not given, the value type is used instead
+     * \returns the expression to evaluate field equality
+     * \deprecated QGIS 3.38. Use the method with a QMetaType::Type argument instead.
+     */
+    Q_DECL_DEPRECATED static QString createFieldEqualityExpression( const QString &fieldName, const QVariant &value, QVariant::Type fieldType ) SIP_DEPRECATED;
 
     /**
      * Returns TRUE if the given \a expression is a simple "field=value" type expression.
@@ -728,11 +815,17 @@ class CORE_EXPORT QgsExpression
 
     QgsExpressionPrivate *d = nullptr;
 
+    static HelpTextHash sFunctionHelpTexts;
+
     //! \note not available in Python bindings
     static void initFunctionHelp() SIP_SKIP;
     //! \note not available in Python bindings
     static void initVariableHelp() SIP_SKIP;
 
+    friend class QgsExpressionNodeFunction;
+    static QRecursiveMutex sFunctionsMutex;
+    static QMap< QString, int> sFunctionIndexMap;
+
     friend class QgsOgcUtils;
 };
 
