@@ -36,17 +36,16 @@ using namespace nlohmann;
 using namespace inja;
 
 
-
 QVariantMap QgsServerOgcApiHandler::values( const QgsServerApiContext &context ) const
 {
-  QVariantMap result ;
+  QVariantMap result;
   const auto constParameters { parameters( context ) };
   for ( const auto &p : constParameters )
   {
     // value() calls the validators and throws an exception if validation fails
     result[p.name()] = p.value( context );
   }
-  const auto sanitizedPath { QgsServerOgcApi::sanitizeUrl( context.handlerPath( ) ).path() };
+  const auto sanitizedPath { QgsServerOgcApi::sanitizeUrl( context.handlerPath() ).path() };
   const auto match { path().match( sanitizedPath ) };
   if ( match.hasMatch() )
   {
@@ -54,8 +53,8 @@ QVariantMap QgsServerOgcApiHandler::values( const QgsServerApiContext &context )
     // Get named path parameters
     for ( const auto &name : constNamed )
     {
-      if ( ! name.isEmpty() )
-        result[name] = QUrlQuery( match.captured( name ) ).toString() ;
+      if ( !name.isEmpty() )
+        result[name] = QUrlQuery( match.captured( name ) ).toString();
     }
   }
   return result;
@@ -99,9 +98,7 @@ QString QgsServerOgcApiHandler::contentTypeForAccept( const QString &accept ) co
     }
   }
   // Log level info because this is not completely unexpected
-  QgsMessageLog::logMessage( QStringLiteral( "Content type for accept %1 not found!" ).arg( accept ),
-                             QStringLiteral( "Server" ),
-                             Qgis::MessageLevel::Info );
+  QgsMessageLog::logMessage( QStringLiteral( "Content type for accept %1 not found!" ).arg( accept ), QStringLiteral( "Server" ), Qgis::MessageLevel::Info );
 
   return QString();
 }
@@ -113,7 +110,7 @@ void QgsServerOgcApiHandler::write( json &data, const QgsServerApiContext &conte
   {
     case QgsServerOgcApi::ContentType::HTML:
       data["handler"] = schema( context );
-      if ( ! htmlMetadata.is_null() )
+      if ( !htmlMetadata.is_null() )
       {
         data["metadata"] = htmlMetadata;
       }
@@ -141,7 +138,7 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
 {
   QUrl url { context.request()->url() };
   QString urlBasePath { context.matchedPath() };
-  const auto match { path().match( QgsServerOgcApi::sanitizeUrl( context.handlerPath( ) ).path( ) ) };
+  const auto match { path().match( QgsServerOgcApi::sanitizeUrl( context.handlerPath() ).path() ) };
   if ( match.captured().count() > 0 )
   {
     url.setPath( urlBasePath + match.captured( 0 ) );
@@ -155,7 +152,7 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
   const auto suffixLength { QFileInfo( url.path() ).suffix().length() };
   if ( suffixLength > 0 )
   {
-    auto path {url.path()};
+    auto path { url.path() };
     path.truncate( path.length() - ( suffixLength + 1 ) );
     url.setPath( path );
   }
@@ -165,7 +162,7 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
 
   // (re-)add extension
   // JSON is the default anyway so we don't need to add it
-  if ( ! extension.isEmpty() )
+  if ( !extension.isEmpty() )
   {
     // Remove trailing slashes if any.
     QString path { url.path() };
@@ -176,24 +173,23 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
     url.setPath( path + '.' + extension );
   }
   return QgsServerOgcApi::sanitizeUrl( url ).toString( QUrl::FullyEncoded ).toStdString();
-
 }
 
 void QgsServerOgcApiHandler::jsonDump( json &data, const QgsServerApiContext &context, const QString &contentType ) const
 {
   // Do not append timestamp to openapi
-  if ( ! QgsServerOgcApi::contentTypeMimes().value( QgsServerOgcApi::ContentType::OPENAPI3 ).contains( contentType, Qt::CaseSensitivity::CaseInsensitive ) )
+  if ( !QgsServerOgcApi::contentTypeMimes().value( QgsServerOgcApi::ContentType::OPENAPI3 ).contains( contentType, Qt::CaseSensitivity::CaseInsensitive ) )
   {
     QDateTime time { QDateTime::currentDateTime() };
     time.setTimeSpec( Qt::TimeSpec::UTC );
-    data["timeStamp"] = time.toString( Qt::DateFormat::ISODate ).toStdString() ;
+    data["timeStamp"] = time.toString( Qt::DateFormat::ISODate ).toStdString();
   }
   context.response()->setStatusCode( 200 );
   context.response()->setHeader( QStringLiteral( "Content-Type" ), contentType );
 #ifdef QGISDEBUG
   context.response()->write( data.dump( 2 ) );
 #else
-  context.response()->write( data.dump( ) );
+  context.response()->write( data.dump() );
 #endif
 }
 
@@ -205,11 +201,8 @@ json QgsServerOgcApiHandler::schema( const QgsServerApiContext &context ) const
 
 json QgsServerOgcApiHandler::link( const QgsServerApiContext &context, const QgsServerOgcApi::Rel &linkType, const QgsServerOgcApi::ContentType contentType, const std::string &title ) const
 {
-  json l
-  {
-    {
-      "href", href( context, "/",
-                    QgsServerOgcApi::contentTypeToExtension( contentType ) )
+  json l {
+    { "href", href( context, "/", QgsServerOgcApi::contentTypeToExtension( contentType ) )
     },
     { "rel", QgsServerOgcApi::relToString( linkType ) },
     { "type", QgsServerOgcApi::mimeType( contentType ) },
@@ -225,29 +218,26 @@ json QgsServerOgcApiHandler::links( const QgsServerApiContext &context ) const
   const QList<QgsServerOgcApi::ContentType> constCts { contentTypes() };
   for ( const auto &ct : constCts )
   {
-    links.push_back( link( context, ( ct == currentCt ? QgsServerOgcApi::Rel::self :
-                                      QgsServerOgcApi::Rel::alternate ), ct,
-                           linkTitle()  + " as " + QgsServerOgcApi::contentTypeToStdString( ct ) ) );
+    links.push_back( link( context, ( ct == currentCt ? QgsServerOgcApi::Rel::self : QgsServerOgcApi::Rel::alternate ), ct, linkTitle() + " as " + QgsServerOgcApi::contentTypeToStdString( ct ) ) );
   }
   return links;
 }
 
 QgsVectorLayer *QgsServerOgcApiHandler::layerFromContext( const QgsServerApiContext &context ) const
 {
-  if ( ! context.project() )
+  if ( !context.project() )
   {
     throw QgsServerApiImproperlyConfiguredException( QStringLiteral( "Project is invalid or undefined" ) );
   }
   // Check collectionId
-  const QRegularExpressionMatch match { path().match( context.request()->url().path( ) ) };
-  if ( ! match.hasMatch() )
+  const QRegularExpressionMatch match { path().match( context.request()->url().path() ) };
+  if ( !match.hasMatch() )
   {
     throw QgsServerApiNotFoundError( QStringLiteral( "Collection was not found" ) );
   }
   const QString collectionId { match.captured( QStringLiteral( "collectionId" ) ) };
   // May throw if not found
   return layerFromCollectionId( context, collectionId );
-
 }
 
 const QString QgsServerOgcApiHandler::staticPath( const QgsServerApiContext &context ) const
@@ -273,14 +263,14 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
 {
   context.response()->setHeader( QStringLiteral( "Content-Type" ), QStringLiteral( "text/html" ) );
   auto path { templatePath( context ) };
-  if ( ! QFile::exists( path ) )
+  if ( !QFile::exists( path ) )
   {
     QgsMessageLog::logMessage( QStringLiteral( "Template not found error: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::MessageLevel::Critical );
     throw QgsServerApiBadRequestException( QStringLiteral( "Template not found: %1" ).arg( QFileInfo( path ).fileName() ) );
   }
 
   QFile f( path );
-  if ( ! f.open( QFile::ReadOnly | QFile::Text ) )
+  if ( !f.open( QFile::ReadOnly | QFile::Text ) )
   {
     QgsMessageLog::logMessage( QStringLiteral( "Could not open template file: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::MessageLevel::Critical );
     throw QgsServerApiInternalServerError( QStringLiteral( "Could not open template file: %1" ).arg( QFileInfo( path ).fileName() ) );
@@ -293,18 +283,16 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
     Environment env { QString( pathInfo.dir().path() + QDir::separator() ).toStdString() };
 
     // For template debugging:
-    env.add_callback( "json_dump", 0, [ = ]( Arguments & )
-    {
+    env.add_callback( "json_dump", 0, [=]( Arguments & ) {
       return data.dump();
     } );
 
     // Path manipulation: appends a directory path to the current url
-    env.add_callback( "path_append", 1, [ = ]( Arguments & args )
-    {
+    env.add_callback( "path_append", 1, [=]( Arguments &args ) {
       auto url { context.request()->url() };
-      QFileInfo fi{ url.path() };
+      QFileInfo fi { url.path() };
       auto suffix { fi.suffix() };
-      auto fName { fi.filePath()};
+      auto fName { fi.filePath() };
       if ( !suffix.isEmpty() )
       {
         fName.chop( suffix.length() + 1 );
@@ -314,7 +302,7 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
       {
         fName.chop( 1 );
       }
-      fName += '/' + QString::fromStdString( args.at( 0 )->get<std::string>( ) );
+      fName += '/' + QString::fromStdString( args.at( 0 )->get<std::string>() );
       if ( !suffix.isEmpty() )
       {
         fName += '.' + suffix;
@@ -325,15 +313,15 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
     } );
 
     // Path manipulation: removes the specified number of directory components from the current url path
-    env.add_callback( "path_chomp", 1, [ = ]( Arguments & args )
-    {
-      QUrl url { QString::fromStdString( args.at( 0 )->get<std::string>( ) ) };
-      QFileInfo fi{ url.path() };
+    env.add_callback( "path_chomp", 1, [=]( Arguments &args ) {
+      QUrl url { QString::fromStdString( args.at( 0 )->get<std::string>() ) };
+      QFileInfo fi { url.path() };
       auto suffix { fi.suffix() };
-      auto fName { fi.filePath()};
+      auto fName { fi.filePath() };
       fName.chop( suffix.length() + 1 );
       // Chomp last segment
-      fName = fName.replace( QRegularExpression( R"raw(\/[^/]+$)raw" ), QString() );
+      const thread_local QRegularExpression segmentRx( R"raw(\/[^/]+$)raw" );
+      fName = fName.replace( segmentRx, QString() );
       if ( !suffix.isEmpty() )
       {
         fName += '.' + suffix;
@@ -345,15 +333,14 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
 
     // Returns filtered links from a link list
     // links_filter( <links>, <key>, <value> )
-    env.add_callback( "links_filter", 3, [ = ]( Arguments & args )
-    {
-      json links = args.at( 0 )->get<json>( );
-      if ( ! links.is_array() )
+    env.add_callback( "links_filter", 3, [=]( Arguments &args ) {
+      json links = args.at( 0 )->get<json>();
+      if ( !links.is_array() )
       {
         links = json::array();
       }
-      std::string key { args.at( 1 )->get<std::string>( ) };
-      std::string value { args.at( 2 )->get<std::string>( ) };
+      std::string key { args.at( 1 )->get<std::string>() };
+      std::string value { args.at( 2 )->get<std::string>() };
       json result = json::array();
       for ( const auto &l : links )
       {
@@ -366,27 +353,24 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
     } );
 
     // Returns a short name from content types
-    env.add_callback( "content_type_name", 1, [ = ]( Arguments & args )
-    {
-      const QgsServerOgcApi::ContentType ct { QgsServerOgcApi::contenTypeFromExtension( args.at( 0 )->get<std::string>( ) ) };
+    env.add_callback( "content_type_name", 1, [=]( Arguments &args ) {
+      const QgsServerOgcApi::ContentType ct { QgsServerOgcApi::contentTypeFromExtension( args.at( 0 )->get<std::string>() ) };
       return QgsServerOgcApi::contentTypeToStdString( ct );
     } );
 
     // Replace newlines with <br>
-    env.add_callback( "nl2br", 1, [ = ]( Arguments & args )
-    {
-      QString text { QString::fromStdString( args.at( 0 )->get<std::string>( ) ) };
+    env.add_callback( "nl2br", 1, [=]( Arguments &args ) {
+      QString text { QString::fromStdString( args.at( 0 )->get<std::string>() ) };
       return text.replace( '\n', QLatin1String( "<br>" ) ).toStdString();
     } );
 
 
     // Returns a list of parameter component data from components -> parameters by ref name
     // parameter( <ref object> )
-    env.add_callback( "component_parameter", 1, [ = ]( Arguments & args )
-    {
+    env.add_callback( "component_parameter", 1, [=]( Arguments &args ) {
       json ret = json::array();
-      json ref = args.at( 0 )->get<json>( );
-      if ( ! ref.is_object() )
+      json ref = args.at( 0 )->get<json>();
+      if ( !ref.is_object() )
       {
         return ret;
       }
@@ -405,9 +389,8 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
 
 
     // Static: returns the full URL to the specified static <path>
-    env.add_callback( "static", 1, [ = ]( Arguments & args )
-    {
-      auto asset( args.at( 0 )->get<std::string>( ) );
+    env.add_callback( "static", 1, [=]( Arguments &args ) {
+      auto asset( args.at( 0 )->get<std::string>() );
       QString matchedPath { context.matchedPath() };
       // If its the root path '/' strip it!
       if ( matchedPath == '/' )
@@ -417,6 +400,12 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
       return matchedPath.toStdString() + "/static/" + asset;
     } );
 
+
+    // Returns true if a string begins with the provided string prefix, false otherwise
+    env.add_callback( "starts_with", 2, []( Arguments &args ) {
+      return string_view::starts_with( args.at( 0 )->get<std::string_view>(), args.at( 1 )->get<std::string_view>() );
+    } );
+
     context.response()->write( env.render_file( pathInfo.fileName().toStdString(), data ) );
   }
   catch ( std::exception &e )
@@ -433,11 +422,11 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
   bool found { false };
   // First file extension ...
   const QString extension { QFileInfo( request->url().path() ).suffix().toUpper() };
-  if ( ! extension.isEmpty() )
+  if ( !extension.isEmpty() )
   {
     static QMetaEnum metaEnum { QMetaEnum::fromType<QgsServerOgcApi::ContentType>() };
     bool ok { false };
-    const int ct  { metaEnum.keyToValue( extension.toLocal8Bit().constData(), &ok ) };
+    const int ct { metaEnum.keyToValue( extension.toLocal8Bit().constData(), &ok ) };
     if ( ok )
     {
       result = static_cast<QgsServerOgcApi::ContentType>( ct );
@@ -450,14 +439,14 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
   }
   // ... then "Accept"
   const QString accept { request->header( QStringLiteral( "Accept" ) ) };
-  if ( ! found && ! accept.isEmpty() )
+  if ( !found && !accept.isEmpty() )
   {
     const QString ctFromAccept { contentTypeForAccept( accept ) };
-    if ( ! ctFromAccept.isEmpty() )
+    if ( !ctFromAccept.isEmpty() )
     {
       const auto constContentTypes( QgsServerOgcApi::contentTypeMimes() );
       auto it = constContentTypes.constBegin();
-      while ( ! found && it != constContentTypes.constEnd() )
+      while ( !found && it != constContentTypes.constEnd() )
       {
         int idx = it.value().indexOf( ctFromAccept );
         if ( idx >= 0 )
@@ -474,7 +463,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
     }
   }
   // Validation: check if the requested content type (or an alias) is supported by the handler
-  if ( ! contentTypes().contains( result ) )
+  if ( !contentTypes().contains( result ) )
   {
     // Check aliases
     bool found { false };
@@ -492,7 +481,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
       }
     }
 
-    if ( ! found )
+    if ( !found )
     {
       QgsMessageLog::logMessage( QStringLiteral( "Unsupported Content-Type: %1" ).arg( QgsServerOgcApi::contentTypeToString( result ) ), QStringLiteral( "Server" ), Qgis::MessageLevel::Info );
       throw QgsServerApiBadRequestException( QStringLiteral( "Unsupported Content-Type: %1" ).arg( QgsServerOgcApi::contentTypeToString( result ) ) );
@@ -506,7 +495,7 @@ QString QgsServerOgcApiHandler::parentLink( const QUrl &url, int levels )
   QString path { url.path() };
   const QFileInfo fi { path };
   const QString suffix { fi.suffix() };
-  if ( ! suffix.isEmpty() )
+  if ( !suffix.isEmpty() )
   {
     path.chop( suffix.length() + 1 );
   }
@@ -514,15 +503,15 @@ QString QgsServerOgcApiHandler::parentLink( const QUrl &url, int levels )
   {
     path.chop( 1 );
   }
-  QRegularExpression re( R"raw(\/[^/]+$)raw" );
-  for ( int i = 0; i < levels ; i++ )
+  const thread_local QRegularExpression re( R"raw(\/[^/]+$)raw" );
+  for ( int i = 0; i < levels; i++ )
   {
     path = path.replace( re, QString() );
   }
   QUrl result( url );
   QUrlQuery query( result );
-  QList<QPair<QString, QString> > qi;
-  const auto constItems { query.queryItems( ) };
+  QList<QPair<QString, QString>> qi;
+  const auto constItems { query.queryItems() };
   for ( const auto &i : constItems )
   {
     if ( i.first.compare( QStringLiteral( "MAP" ), Qt::CaseSensitivity::CaseInsensitive ) == 0 )
@@ -531,7 +520,7 @@ QString QgsServerOgcApiHandler::parentLink( const QUrl &url, int levels )
     }
   }
   // Make sure the parent link ends with a slash
-  if ( ! path.endsWith( '/' ) )
+  if ( !path.endsWith( '/' ) )
   {
     path.append( '/' );
   }
@@ -554,30 +543,9 @@ QgsVectorLayer *QgsServerOgcApiHandler::layerFromCollectionId( const QgsServerAp
 
 json QgsServerOgcApiHandler::defaultResponse()
 {
-  static json defRes =
-  {
+  static json defRes = {
     { "description", "An error occurred." },
-    {
-      "content", {
-        {
-          "application/json", {
-            {
-              "schema", {
-                { "$ref", "#/components/schemas/exception" }
-              }
-            }
-          }
-        },
-        {
-          "text/html", {
-            {
-              "schema", {
-                { "type", "string" }
-              }
-            }
-          }
-        }
-      }
+    { "content", { { "application/json", { { "schema", { { "$ref", "#/components/schemas/exception" } } } } }, { "text/html", { { "schema", { { "type", "string" } } } } } }
     }
   };
   return defRes;
